/*!
 * \file   include/TFEL/Math/Vector/VectorConcept.hxx
 * \author Helfer Thomas
 * \copyright Copyright (C) 2006-2014 CEA/DEN, EDF R&D. All rights 
 * reserved. 
 * This project is publicly released under either the GNU GPL Licence 
 * or the CECILL-A licence. A copy of thoses licences are delivered 
 * with the sources of TFEL. CEA or EDF may also distribute this 
 * project under specific licensing conditions. 
 */

#ifndef _VECTOR_CONCEPT_LIB_
#define _VECTOR_CONCEPT_LIB_ 1

#include<string>

#include"TFEL/Config/TFELConfig.hxx"

#include"TFEL/Metaprogramming/InvalidType.hxx"
#include"TFEL/Metaprogramming/IF.hxx"
#include"TFEL/TypeTraits/IsTemporary.hxx"

#include"TFEL/Math/General/ResultType.hxx"
#include"TFEL/Math/General/BasicOperations.hxx"

#include"TFEL/Math/Forward/VectorConcept.hxx"

namespace tfel{

  namespace math{
    
    template<typename T>
    struct VectorTraits{
      typedef tfel::meta::InvalidType NumType;
      typedef tfel::meta::InvalidType IndexType;
      typedef tfel::meta::InvalidType RunTimeProperties;
    };

    /*!
     * \class VectorTag
     * \brief Helper class to characterise vectors.
     */ 
    struct VectorTag
    {
      /*!
       * \brief  Return the name of the class.
       * \param  void.
       * \return std::string, the name of the class.
       * \see    Name.
       */
      static std::string 
      getName(void);

    }; // end of VectorTag
    
    template<typename T>
    struct VectorConcept 
    {

    private:

      typedef VectorTraits<T> traits;
      static const bool isTemporary = tfel::typetraits::IsTemporary<T>::cond;
      typedef typename tfel::meta::IF<isTemporary,
				      typename traits::NumType,
				      const typename traits::NumType&>::type ValueType;

    protected:
      ~VectorConcept(){};
      
    public :
    
      typedef VectorTag ConceptTag;
        
      TFEL_MATH_INLINE 
      ValueType
      operator()(const typename traits::IndexType) const ;

      TFEL_MATH_INLINE 
      ValueType
      operator[](const typename traits::IndexType) const ;
      
    };

    template<typename T>
    struct VectorType
    {
      typedef T type;
    };
  
  } // end of namespace math

} // end of namespace tfel

#include"TFEL/Math/Vector/VectorConcept.ixx"

#endif /* _VECTOR_CONCEPT_LIB_ */
