/*!
 * \file   mfront/include/MFront/Aster/AsterTraits.hxx
 * \brief  
 * 
 * \author Helfer Thomas
 * \date   13 fv 2007
 * \copyright Copyright (C) 2006-2014 CEA/DEN, EDF R&D. All rights 
 * reserved. 
 * This project is publicly released under either the GNU GPL Licence 
 * or the CECILL-A licence. A copy of thoses licences are delivered 
 * with the sources of TFEL. CEA or EDF may also distribute this 
 * project under specific licensing conditions. 
 */

#ifndef _LIB_MFRONT_ASTERTRAITS_HXX_
#define _LIB_MFRONT_ASTERTRAITS_HXX_ 

namespace aster{

  enum AsterBehaviourType{
    SMALLSTRAINSTANDARDBEHAVIOUR,
    FINITESTRAINSTANDARDBEHAVIOUR,
    COHESIVEZONEMODEL
  }; // end of enum ASTERBehaviourType

  enum AsterSymmetryType{
    ISOTROPIC,
    ORTHOTROPIC
  }; // end of enum AsterSymmetryType

  enum AsterErrorReportPolicy{
    ASTER_WRITEONSTDOUT,
    ASTER_NOERRORREPORT
  }; // end of enum AsterErrorReportPolicy

  template<typename Behaviour>
  struct AsterTraits{
    static const AsterBehaviourType btype = aster::SMALLSTRAINSTANDARDBEHAVIOUR;
    // size of the driving variable array (STRAN)
    static const unsigned short DrivingVariableSize = 0u;
    // size of the thermodynamic force variable array (STESS)
    static const unsigned short ThermodynamicForceVariableSize = 0u;
    static const AsterErrorReportPolicy errorReportPolicy = ASTER_WRITEONSTDOUT;
    static const bool requiresStiffnessTensor             = false;
    static const bool requiresThermalExpansionCoefficientTensor = false;
    static const unsigned short propertiesOffset          = 0u;
    // Type of the behaviour of the material (isotropic or orthotropic)
    static const AsterSymmetryType type   = aster::ISOTROPIC;
    // Type of the elastic behaviour of the material (isotropic or orthotropic)
    static const AsterSymmetryType  etype = aster::ISOTROPIC;
  }; // end of class ASTERTraits

} // end of namespace aster  

#endif /* _LIB_MFRONT_ASTERTRAITS_HXX */

