/*!
 * \file   mfront/tests/properties/interfaces/include/MFront/MFrontPleiadesLawInterface-2.0.hxx
 * \brief  This file declares the MFrontPleiadesLawInterface class
 * \author Helfer Thomas
 * \date   06 mai 2008
 * \copyright Copyright (C) 2006-2014 CEA/DEN, EDF R&D. All rights 
 * reserved. 
 * This project is publicly released under either the GNU GPL Licence 
 * or the CECILL-A licence. A copy of thoses licences are delivered 
 * with the sources of TFEL. CEA or EDF may also distribute this 
 * project under specific licensing conditions. 
 */

#ifndef _LIB_MFRONTPLEIADESLAWINTERFACE_H_
#define _LIB_MFRONTPLEIADESLAWINTERFACE_H_ 

#include<string>
#include<fstream>

#include"TFEL/Utilities/CxxTokenizer.hxx"

#include"MFront/VariableDescription.hxx"
#include"MFront/StaticVariableDescription.hxx"
#include"MFront/LawFunction.hxx"
#include"MFront/VariableBoundsDescription.hxx"
#include"MFront/MFrontLawVirtualInterface.hxx"

namespace mfront{

class MFrontPleiadesLawInterface : public MFrontLawVirtualInterface {
  public:

  MFrontPleiadesLawInterface();
  ~MFrontPleiadesLawInterface();

  static std::string getName(void);

  void setVerboseMode(void);
  void setDebugMode(void);
  void setWarningMode(void);

  /*!
   * \param const std::string&, library
   * \param const std::string&, material
   * \param const std::string&, class
   */
  std::map<std::string,std::vector<std::string> >
  getGlobalIncludes(const std::string&,
                    const std::string&,
                    const std::string&);

  /*!
   * \param const std::string&, library
   * \param const std::string&, material
   * \param const std::string&, class
   */
  std::map<std::string,std::vector<std::string> >
  getGlobalDependencies(const std::string&,
                        const std::string&,
                        const std::string&);

  /*!
   * \param const std::string&, library
   * \param const std::string&, material
   * \param const std::string&, class
   */
  std::map<std::string,std::vector<std::string> >
  getGeneratedSources(const std::string&,
                      const std::string&,
                      const std::string&);

  /*!
   * \param const std::string&, library
   * \param const std::string&, material
   * \param const std::string&, class
   */
  std::vector<std::string>
  getGeneratedIncludes(const std::string&,
                       const std::string&,
                       const std::string&);

  /*!
   * \param const std::string&, library
   * \param const std::string&, material
   * \param const std::string&, class
   */
  std::map<std::string,std::vector<std::string> >
  getLibrariesDependencies(const std::string&,
                           const std::string&,
                           const std::string&);

  /*!
   * \param const std::string&, library
   * \param const std::string&, material
   * \param const std::string&, class
   * \param const std::vector<std::string>&, library links
   */
  std::map<std::string,
  std::pair<std::vector<std::string>,
  std::vector<std::string> > >
  getSpecificTargets(const std::string&,
                     const std::string&,
                     const std::string&,
                     const std::vector<std::string>&);

  std::pair<bool,tfel::utilities::CxxTokenizer::TokensContainer::const_iterator>
  treatKeyword(const std::string&,
               tfel::utilities::CxxTokenizer::TokensContainer::const_iterator,
               const tfel::utilities::CxxTokenizer::TokensContainer::const_iterator);

  /*
   * \param const std::string&, name of the original file
   * \param const std::string&, name of the output library
   * \param const std::string&, name of the material
   * \param const std::string&, name of the class
   * \param const std::string&, author
   * \param const std::string&, date
   * \param const std::string&, description
   * \param const std::string&, includes
   * \param const std::string&, output name
   * \param const VarContainer&, inputs
   * \param const std::vector<std::string>&, material laws
   * \param const std::map<std::string,std::string>&, glossary names,
   * \param const std::map<std::string,std::string>&, entry names,
   * \param const StaticVarContainer&, static variables
   * \param const std::vector<std::string>&, parameters
   * \param const std::map<std::string,double>&, parameters values
   * \param const LawFunction&, function definition
   * \param const std::vector<VariableBoundsDescription>&, bounds of the law
   * \param const std::vector<VariableBoundsDescription>&, physical bounds of the law
   * \param const bool, use template
   * \param const std::vector<std::string>&, namespaces
   */
  void writeOutputFiles(const std::string&,
                        const std::string&,
                        const std::string&,
                        const std::string&,
                        const std::string&,
                        const std::string&,
                        const std::string&,
                        const std::string&,
                        const std::string&,
                        const VarContainer&,
                        const std::vector<std::string>&,
                        const std::map<std::string,std::string>&,
                        const std::map<std::string,std::string>&,
                        const StaticVarContainer& staticVars,
                        const std::vector<std::string>&,
                        const std::map<std::string,double>&,
                        const LawFunction&,
                        const std::vector<VariableBoundsDescription>&,
                        const std::vector<VariableBoundsDescription>&,
                        const bool,
                        const std::vector<std::string>&);

  void reset(void);

  private:

  /*
   * \param const std::string&, name of the original file
   * \param const std::string&, className
   * \param const std::string&, author
   * \param const std::string&, date
   * \param const std::string&, description
   * \param const std::string&, includes
   * \param const VarContainer&, inputs
   */
  void writeHeaderFile(const std::string&,
                       const std::string&,
                       const std::string&,
                       const std::string&,
                       const std::string&,
                       const std::string&,
                       const VarContainer&);

  /*
   * \param const std::string&, name of the original file
   * \param const std::string&, className
   * \param const std::string&, author
   * \param const std::string&, date
   * \param const std::string&, output name
   * \param const VarContainer&, inputs
   * \param const std::vector<std::string>&, material laws
   * \param const std::map<std::string,std::string>&, glossary names,
   * \param const std::map<std::string,std::string>&, entry names,
   * \param const StaticVarContainer&, static variables
   * \param const std::vector<std::string>&, parameters
   * \param const std::map<std::string,double>&, parameters values
   * \param const LawFunction&, function definition
   * \param const std::vector<VariableBoundsDescription>&, bounds of the law
   * \param const std::vector<VariableBoundsDescription>&, physical bounds of the law
   */
  void writeSrcFile(const std::string&,
                    const std::string&,
                    const std::string&,
                    const std::string&,
                    const std::string&,
                    const VarContainer&,
                    const std::vector<std::string>&,
                    const std::map<std::string,std::string>&,
                    const std::map<std::string,std::string>&,
                    const StaticVarContainer& staticVars,
                    const std::vector<std::string>&,
                    const std::map<std::string,double>&,
                    const LawFunction&,
                    const std::vector<VariableBoundsDescription>&,
                    const std::vector<VariableBoundsDescription>&);

  std::ofstream headerFile;
  std::ofstream srcFile;
  std::string headerFileName;
  std::string srcFileName;
  bool verboseMode;
  bool debugMode;
  bool warningMode;
}; // end of MfrontPleiadesLawInterface

} // end of namespace mfront

#endif /* _LIB_MFRONTPLEIADESLAWINTERFACE_H */
