#include <unistd.h>
#include <sys/types.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include<math.h>

#ifdef WIN32
#include <windows.h>
#include <tchar.h>
#include <stdio.h>
#define dlopen(name, mode) LoadLibrary (TEXT (name))
#define dlsym(handle, func) GetProcAddress (handle, func)
#define dlclose(handle) FreeLibrary (handle)
#define dlerror() "error %d occurred", GetLastError ()
typedef double (*MatFctPtr)(const double *const);
#else
#include <dlfcn.h>
typedef double (*MatFctPtr)(const double *const);
#endif

#include "castem.h"
#ifdef dlclose
#undef dlclose
#endif
#ifdef dlopen
#undef dlopen
#endif
#ifdef dlsym
#undef dlsym
#endif

/*
 *     DESCRIPTION DE L'INTERFACE :
 *     --------------------------
 *     ENTREE : VALPAR : REAL*8(NPARA), valeurs des parametres
 *              NPARA  : INTEGER, nombre de parametres
 *              NLOI : Numro de la loi
 *     SORTIE : VALCMP : REAL*8, valeur de la composante
 *              IERUT  : INTEGER, code retour
 *                       IERUT= 0 : OK
 *                       IERUT<>0 : NOOK
 *                       Les cas d'erreur pris en charge par la
 *                       programmation de l'utilisateur doivent
 *                       faire l'objet d'une affectation de IERUT
 *                       differente de 0
 */
/* debut a decommenter*/
void loiexterne(double *valpar,
		double *valcmp,
		MatFctPtr* loi,
		cast_int *ierut)
{
  *valcmp = (*loi)(valpar);
  if(isnan(*valcmp)){
    *ierut=2;
  }
}
/*fin a decommenter */

/*
 * Rcupre le numro de la loi Pliades en fonction de son nom
 * Le numro correspond  son rang dans le fichier LOI.inc
 *
 * mmod : Nom du modle
 * lm : longueur de la chane de caractres prcdente
 * mlib : Nom de la librairie
 * ll : longueur de la chane de caractres prcdente
 * nomco4 : Nom du paramtre  valuer
 * nompar : Noms des paramtres de la liste d'entre
 * npara : Nombre de paramtres
 * lib : Pointeur sur la librairie dynamique ouverte
 * nloi : numro du modle
 * En cas d'erreur, le modle retourn est nul
 */
/* debut a decommenter*/
void numloi(char *const mmod,cast_int *lm,char *mlib,cast_int *ll,
	    char *const unused1,char *unused2,
	    cast_int *const npara,void**lib,MatFctPtr* nloi,
	    const size_t unused3,const size_t unused4,
	    const size_t unused5,const size_t unused6)
{
  char * modele;
  char * librairie;
  unsigned short *n;

  (void) unused1;
  (void) unused2;
  (void) unused3;
  (void) unused4;
  (void) unused5;
  (void) unused6;

  modele = (char *) malloc((*lm+7)*sizeof(char));

  if(!modele){
    fprintf(stderr,"mmoire insuffisante\n");
    exit(EXIT_FAILURE);
  }
  strncpy(modele,mmod,(unsigned int)*lm);
  modele[*lm]='\0';
  /*fin a decommenter */
  /* Initialisation  0 du numro de modle */
  /* debut a decommenter*/
  *nloi=0;

  if (*ll) {
    librairie  = (char *) malloc((*ll+1)*sizeof(char));
    if(!librairie){
      fprintf(stderr,"mmoire insuffisante\n");
      exit(EXIT_FAILURE);
    }
    librairie[*ll]='\0';
    strncpy(librairie,mlib,(unsigned int)*ll);
    *lib= (void *) dlopen (librairie, RTLD_LAZY);
    free(librairie);
  }
  else {
    *lib = (void *) dlopen (NULL, RTLD_LAZY);
  }
  if (!(*lib)){
    fprintf(stderr,"%s\n",dlerror());
    return;
  }
  /*fin a decommenter */
  /* Conversion des chanes de caractres en string */
  /* debut a decommenter*/
  *nloi=(MatFctPtr)dlsym(*lib,modele);

  if (!(*nloi)) {
    fprintf(stderr,"%s\n",dlerror());
    return;
  }
  /*fin a decommenter */
  /* Vrification du nombre de paramtres */
  /* debut a decommenter*/
  modele[*lm]='_';
  modele[*lm+1]='n';
  modele[*lm+2]='a';
  modele[*lm+3]='r';
  modele[*lm+4]='g';
  modele[*lm+5]='s';
  modele[*lm+6]='\0';

  n=(unsigned short *) dlsym(*lib,modele);
  if (!n) {
    fprintf(stderr,"%s\n",dlerror());
    dlclose(*lib);
    return;
  }

  if ( *n != (unsigned short) *npara) {
    *nloi=0;
    fprintf(stderr,"Erreur lors de l'valuation du modle %s\n",modele);
    fprintf(stderr,"Nombre de paramtres reus  : %d\n",*npara);
    fprintf(stderr,"Nombre de paramtres requis : %d\n",*n);
    dlclose(*lib);
  }
  free(modele);
}

/*fin a decommenter */
/* Fermeture de la librairie */
/* debut a decommenter*/
void dlclose_(void **lib) {
  dlclose(*lib);
}
/*fin a decommenter */
